/*
 * Esempio di utilizzo delle funzioni del modulo per la visualizzazione di una
 * semplice mappa a scomparsa tramite la Google Maps Javascript API:
 *
 *   <a onclick="handleMap(<REF_ID>, <MAP_LAT>, <MAP_LNG>, '<DESCRIPTION>')" class="MapHandler">
 *     <span id="show_<REF_ID>">Mappa</span><span id="hide_<REF_ID>" style="display:none">Chiudi</span>
 *   </a>
 *   <div id="container_<REF_ID>" style="display:none">
 *     <div id="map_<REF_ID>" class="SellpointMap"></div>
 *   </div>
 *
 * Lo stile del div della mappa, map_<REF_ID>, si puo' impostare con una classe
 * ma non con l'attributo style che verra' impostato da Google al momento della
 * generazione della mappa.
 *
 * NB - Ricordare di caricare la Google Maps Javascript API nella pagina una
 * volta sola:
 *
 *   <script src="https://maps.googleapis.com/maps/api/js?key=<MAPS_API_KEY>"></script>
 */


/**
 * Crea una mappa con un singolo marcatore centrandola sul marcatore.
 *
 * @param Object box Contenitore della mappa.
 * @parma Float lat Latitudine nord del marcatore.
 * @param Float lng Longitudine est del marcatore.
 * @param String title Titolo del marcatore.
 */
function initSimpleMap(box, lat, lng, title) {
  var location = {lat: lat, lng: lng};

  var map = new google.maps.Map(box, {
    center: location,
    zoom: 15
  });

  var marker = new google.maps.Marker({
    position: location,
    map: map,
    title: title
  });
}


/**
 * Visualizza o nasconde la mappa creandola nel caso di visulizzazione se ancora
 * non esiste.
 *
 * @param String id Identificativo univoco della mappa.
 * @parma Float lat Latitudine nord del marcatore.
 * @param Float lng Longitudine est del marcatore.
 * @param String title Titolo del marcatore.
 */
function handleMap(id, lat, lng, title) {
  var $container  = $('container_' + id);
  var $show_label = $('show_' + id);
  var $hide_label = $('hide_' + id);

  if ($container.is(':visible')) {
    // NB : Le Google Maps Javascript API si aspettano di creare la mappa in un
    // vero e proprio DIV, quindi in un elemento HTML con display=block. Devo
    // visualizzare il contenitore della mappa prima di creare la mappa perche'
    // altrimenti se creo piu' mappe nella stessa pagina viene disegnata solo
    // la prima e le successive no.
    $container.show();
    $show_label.hide();
    $hide_label.css('display','inline');

    var $map = $('map_' + id);

    if ($map.innerHTML == '')
      initSimpleMap($map, lat, lng, title);
  }
  else {
    $container.hide();
    $show_label.css('display','inline');
    $hide_label.hide();
  }
}

//Devo aspettare il caricamento del js asincrono
$(window).on('load', function () {
	$('.hasMap').each(function(i) {
	      initSimpleMap($(this).find('.TheatreMap')[0], parseFloat($(this).data('lat')), parseFloat($(this).data('lng')), $(this).data('title'));
	});
});