/**
 * classe che definisce gli elementi selezionati
 */
var Selection = new function() {
	this.items = new Array();
	this.listeners = new Array();
	this.maxTot = 0;
	this.maxOther = 0;
	this.zoneLimit = new Array();
	this.extras = new Array();
	this.reductionLimit = new Array();
	this.reductions = new Array();
	
	/**
	 * aggiunge un elemento alla selezione
	 * item è un oggetto che arriva dall'SVG
	 */
	this.add = function(item) {
		var seat = item;
		if (Selection.contains(seat.id) === false) {
			
			// verifico il vincolo sul numero massimo
			if (this.maxTot != 0 && this.maxTot == this.items.length) {
				// sparo l'evento di numero di posti raggiunto
				var event = new SelectionEvent('maxseats',seat);
				this.fireEvent(event);
				return;
			}
		
			// verifico se sulla zona ci sono delle limitazioni
			if (this.zoneLimit.length > 0) {
				var limit = null;
				// cerco il limite
				for (var i=0; i<this.zoneLimit.length; i++) {
					if (this.zoneLimit[i].zone == seat.zone) {
						limit = this.zoneLimit[i];
					}
				}
				
				var max = 999999;
				if (!limit) {
					// verifico se è impostato other
					if (this.maxOther > 0) {
						max = this.maxOther;
					}
				} else {
					max = limit.max
				}
				
				// verifico se il limite sulla zona è rispettato
				if (max == this.countByZone(seat.zone)) {
					// sparo l'evento di numero di posti raggiunto
					var event = new SelectionEvent('maxseatszone',item);
					this.fireEvent(event);
					return;
				}

			}
									
			// verifico se sulla riduzione ci sono delle limitazioni
			if (this.reductionLimit.length > 0) {
				var limit = null;
				
				// cerco il limite
				for (var i=0; i<this.reductionLimit.length; i++) {
					if (this.reductionLimit[i].reduction == Seats.findReduction(Seats.zones[seat.zone].getDefaultPrice().reduction).id) {
						limit = this.reductionLimit[i];
					}
				}
				
				if (limit) {
					// verifico se il limite sulla zona è rispettato
					if (limit.max == this.countByReduction(limit.reduction)) {
						// sparo l'evento di numero di posti raggiunto
						var event = new SelectionEvent('maxseatszone',item);
						this.fireEvent(event);
						return;
					}
					
				}
				
			}						
			
			// aggiungo l'elemento
			Selection.items.push(item);
			
			// aggiungo la riduzione
			var redu = {};
			redu.seat = item.id;
			redu.reduction = Seats.findReduction(Seats.zones[seat.zone].getDefaultPrice().reduction).id;
			Selection.reductions.push(redu);
						
			// sparo l'evento di posto aggiornato verso chi ascolta
			var event = new SelectionEvent('add', item);			
			this.fireEvent(event);
	
		
		} else {
			this.remove(item.id);
		}
	}
	
	this.fireEvent = function(item) {
		for (var i = 0; i < this.listeners.length; i++) {
			this.listeners[i].changed(item);
		}
	}
	
	this.addListener = function(listener) {
		this.listeners.push(listener);
	}
	
	/**
	 * imposta il massimo per riduzione
	 */
	this.addReductionLimit = function(reduction, max) {
		var limit = {};
		limit.reduction = reduction;
		limit.max = max;
		this.reductionLimit.push(limit);
	}
	
	/**
	 * toglie un elemento dalla selezione
	 */
	this.remove = function(id) {
		pos = this.contains(id);
		if (pos !== false) {

			// sparo l'evento di posto eliminato verso chi ascolta			
			item = this.items[pos];
			var event = new SelectionEvent('remove', item);		
			
			// elimino l'elemento
			this.items.splice(pos,1);	
			
			this.removeFromReductions(id);
			
			this.fireEvent(event);
				
		}
	}
	
	/**
	 * aggiorna la riduzione legata a un seat
	 */
	this.updateReduction = function(seat, reductionIndex) {
		for (var i=0; i< this.reductions.length; i++) {
			if (this.reductions[i].seat == seat) {
				this.reductions[i].reduction = Seats.reductions[reductionIndex].id;
				return;
			}
		}
	}
	
	/**
	 * verifica se un elemento è contenuto nella selezione
	 * restituisce false se non è contenuto, altrimenti l'indice all'interno dell'array
	 */
	this.contains = function(id) {
		for (var i=0; i<Selection.items.length; i++) {
			if (Selection.items[i].id == id) {
				return i;
			}
		}
		return false;
	}
	
	/**
	 * funzione temporanea per svuotare l'elenco dei listener
	 */
	this.clear = function() {
		this.listeners = new Array();
	}
	
	/**
	 * setta l'elemento attualmente visualizzato
	 */
	this.current = function(item) {
		var event = new SelectionEvent((item ? 'select' : 'deselect'), item);
		this.fireEvent(event);
	}
	
	/**
	 * richiama l'elemento attualmente visualizzato
	 */
	this.recall = function(item) {
		
		var event = new SelectionEvent('recall', item);
		this.fireEvent(event);
	}
	
	/**
	 * seleziona il bookmark
	 */
	this.bookmark = function(item) {
		var event = new SelectionEvent('sector', item);
		this.fireEvent(event);
	}
	
	/**
	 * restituisce il numero di elementi selezionati per zona
	 */
	this.countByZone = function(zone) {
		cnt = 0;
		for (var i=0; i<this.items.length; i++) {
			if (Seats.seats[this.items[i].index].zone == zone) {
				cnt++;
			}
		}
		return cnt;
	}
	
	/**
	 * restituisce il numero di elementi selezionati per riduzione
	 */
	this.countByReduction = function(reduction) {
		cnt = 0;
		for (var i=0; i<this.reductions.length; i++) {
			if (this.reductions[i].reduction == reduction) {
				cnt++;
			}
		}
		return cnt;
 
	}
	
	/**
	 * rimuove un seat dalle riduzioni selezionate
	 */
	this.removeFromReductions = function(seat) {
		var pos = -1;
		for (var i=0; i<this.reductions.length; i++) {
			if (this.reductions[i].seat == seat) {
				pos = i;
			}
		}
		if (pos >= 0) {
			this.reductions.splice(pos,1);
		}
	}
	
	/**
	 * mostra l'alert che dice che servono almeno N posti per la riduzione
	 */
	this.warning = function(num) {
		var event = new SelectionEvent('missingseatsforreduction', num);
		this.fireEvent(event);
	}
}

/**
 * evento di modifica della selezione
 */
var SelectionEvent = function(action, item) {
	this.action = action;	
	this.target = item;
	
}
