
	/** classe Map
	 *  gestisce il caricamento e le interazioni sulla mappa
	 */ 
	var MapDrawing = new function() {
		/*
		this.availableSeatsLabel = 'Posti disponibili: ';
		*/
		
		this.interactive = true;
		this.hasminimap = true;
		this.hasroomscreenshot = false;
		
		/**
		 * oggetto SVG
		 */
		this.draw = null;
		/**
		 * etichette informative dei posti 
		 */
		this.labels = null;
		
		/**
		 * livello di zoom
		 * minimo 1
		 * massimo 10?
		 */
		this.zoom = 1;
		
		this.maxzoom = 8;
		this.minzoom = 1;
		this.scaleFactor = 1;
		this.switchFactor = 3;
		this.zoomINfactor = 1.25 ;		
		this.zoomOUTfactor = 0.8 ;
		this.zoomPlusFactor = 1.5;
		this.zoomMinusFactor = 0.666;
		this.finalZoom = 0;
		
		// fattori di correzione per mappa non in 4:3
		this.adjustX = 0;
		this.adjustY = 0;
		
		this.zoomStyle = 'svg';
		
		/**
		 * parametri assoluti per le dimensioni mappa
		 */
		
		//this.abs_width = 640;
		//this.abs_height = 720;
		//this.break_width = 768;
		this.abs_width = 1000;
		this.abs_height = 750;
		this.break_width = 768;
		this.switch_width = 1024;
		
		
		this.width = 640;
		this.height = 480;
		this.svgWidth = 640;
		this.svgHeight = 480;
		
		this.centerX = 320;
		this.centerY = 240;
		this.startX = 0;
		this.startY = 0;
		
		this.orientation = window.orientation;
		
		this.labelW = 220;
		this.labelH = 52;
		this.labelPop = 16;
		this.labelFont = 12;		
		this.labelPOffset = 90;
		
		this.alertPadding = 35;
		
		this.currentseat = null;
		
		this.isDragging = false;
		
		this.isShowingLayer = false;
		
		this.selectedColor = '#FF0000';
		this.hiliteColor = '#EE0000';
		this.inactiveColor = '#404040';
		this.restrictedColor = '#D3CDCD';
		this.hoverColor = '#A1A1A1';
		
		this.minifactor = 6.2;
	
		this.promising = false;
		this.delay = false;
		/**
		 * funzione temporanea, mi serve per resettare le impostazioni
		 */
		this.reset = function() {
			this.draw = null;
			this.zoom = 1;
			
			this.maxzoom = 8;
			this.minzoom = 1;
			this.scaleFactor = 1;
			this.switchFactor = 3;
			
			this.width = 640;
			this.height = 480;
			this.svgWidth = 640;
			this.svgHeight = 480;
			
			this.centerX = 320;
			this.centerY = 240;
			this.startX = 0;
			this.startY = 0;
			
			this.currentseat = null;
			
			this.isDragging = false;
		};
		
		/**
		 * effettua il caricamento iniziale della mappa 
		 */
		this.init = function() {
			
			// aggancio gli eventi di resize e rotazione dello schermo
			window.addEventListener("resize", MapDrawing.checkOrientation, false);
			window.addEventListener("orientationchange", MapDrawing.checkOrientation, false);
			
			this.zoom = this.minzoom;			
						
			// imposto le dimensioni
			this.initSize();
			
			// creo il SVG con lo sfondo e i posti a sedere
			this.paint();
						
			// inizializzo gli eventi
			this.initInputs();
			
			if (this.interactive) {
				// faccio il bind alla selection
				Selection.addListener(this);				
			}
			
			if (this.hasminimap) {
				this.toCanvas();
			}
			
			// aggiungo il layer con la suggestion per il wheel
			this.addZoomLayer();
			
		};
		
		/**
		 * inizializza gil eventi sulla mappa
		 */
		this.initInputs = function() {
			
			// aggancio il mouse scroll per fare lo zoom
			$('#map').bind('wheel', function(e) {
				if (e.ctrlKey) {
					e.preventDefault();
					if (Firefox) {
						if (e.originalEvent.deltaY != undefined) {
							// Firefox
							if (e.originalEvent.deltaY < 0) {
								MapDrawing.zoomIn(MapDrawing.zoomINfactor, e);
							} else {
								MapDrawing.zoomOut(MapDrawing.zoomOUTfactor, e);
							}
						}
							
					} else {
							
						if (e.originalEvent.wheelDelta) {
							// Chrome
							if (e.originalEvent.wheelDelta > 0) {
								MapDrawing.zoomIn(MapDrawing.zoomINfactor, e);
							} else {
								MapDrawing.zoomOut(MapDrawing.zoomOUTfactor, e);
							}
						}
					}
				} else {
					// messaggio per premere CTRL
					if (!MapDrawing.isShowingLayer) {
						$('#map-alert').fadeIn();
						MapDrawing.isShowingLayer = true;
						setTimeout(function() { 
							$('#map-alert').fadeOut();
							MapDrawing.isShowingLayer = false;
						}, 1500);
					}
				}
				
			});
			
			
			

			if (Mobile()) {
				// eventi touch
				$("#map").on('touchstart',function(e) {
					e.preventDefault();
					if (MapDrawing.zoom > MapDrawing.minzoom) {
						MapDrawing.isDragging = true;
						MapDrawing.dragStartX = e.originalEvent.touches[0].pageX;
						MapDrawing.dragStartY = e.originalEvent.touches[0].pageY;
						$(this).css('cursor','grabbing');
					}
				}).on('touchmove',function(e) {	
					//e.preventDefault();
					if (MapDrawing.isDragging) {					
						x = ((MapDrawing.dragStartX - e.originalEvent.touches[0].pageX) ) ;
						y = ((MapDrawing.dragStartY - e.originalEvent.touches[0].pageY) ) ;
						MapDrawing.dragStartX = e.originalEvent.touches[0].pageX;
						MapDrawing.dragStartY = e.originalEvent.touches[0].pageY;					
						MapDrawing.move(x,y);
						$(this).css('cursor','move');
					}
				});
				$(window).on('touchend',function(e) {
					//e.preventDefault();
					MapDrawing.isDragging = false;		
					$("#map").css('cursor','default');
				});
			} else {
				// aggancio il pick & move per muovere la mappa
				$("#map").bind('mousedown',function(e) {
					e.preventDefault();
					if (MapDrawing.zoom > MapDrawing.minzoom) {
						MapDrawing.isDragging = true;
						MapDrawing.dragStartX = e.clientX;
						MapDrawing.dragStartY = e.clientY;
						$(this).css('cursor','grabbing');
					}
				}).on('mousemove',function(e) {			
					e.preventDefault();
					if (MapDrawing.isDragging) {					
						x = ((MapDrawing.dragStartX - e.clientX) ) ;
						y = ((MapDrawing.dragStartY - e.clientY) ) ;
						MapDrawing.dragStartX = e.clientX;
						MapDrawing.dragStartY = e.clientY;					
						MapDrawing.move(x,y);
						$(this).css('cursor','move');
					}
				});
				
				$(window).on('mouseup',function(e) {
					e.preventDefault();
					MapDrawing.isDragging = false;		
					$("#map").css('cursor','default');
				});				
			}
			
			
			// aggancio il click sui pulsanti del navigator per gestire lo zoom
			$('#btn-zoom-in').on({ 
				pointerdown: function(ev) {
					ev.preventDefault();
					MapDrawing.zoomIn();
				}, mousedown: function(ev) {					
					ev.preventDefault();
					MapDrawing.zoomIn();
				}				
			});
			
			$('#btn-zoom-out').on({
				pointerdown: function(ev) {
					ev.preventDefault();
					MapDrawing.zoomOut();
				}, mousedown: function(ev) {
					ev.preventDefault();
					MapDrawing.zoomOut();
				}
			});
			
			$('#btn-zoom-reset').on({
				pointerdown: function(ev) {
					ev.preventDefault();
					MapDrawing.zoomRestore();
				}, mousedown: function(ev) {
					ev.preventDefault();
					MapDrawing.zoomRestore();
				}
			});
		};
		
		this.addZoomLayer = function() {
			var layer = document.createElement('div');
			layer.style = 'position:absolute; width:100%; height:100%; opacity:0.66; background-color: #000; color:#FFF; left:0px; top:0px;  font-size:2.2rem; padding-top:' +
				MapDrawing.alertPadding +'%; display:none; text-align:center;';
			layer.innerHTML = frasi.mappa['lbl-scroll'];
			layer.id = 'map-alert';
			
			$('#map').append(layer);
		};
		
		/**
		 * effettua lo zoom in sulla mappa
		 */
		this.zoomIn = function(value, e) {
			
			if (value === undefined) { value = this.zoomPlusFactor; }
			if (e === undefined) { e = null; }
			
			if (this.zoom < this.maxzoom && !this.promising) {
				var oldZoom = this.zoom;
				
				if (this.zoom * value < this.maxzoom) {
					this.zoom *= value;
				} else {
					this.zoom = this.maxzoom;
				}
				// se ho passato l'evento del mouse 
				// tengo fisse le coordinate del mouse e sposto il centro di conseguenza
				
				if (e && e.originalEvent) {
					
					// calcolo le coordinate scalate
					var realX = e.originalEvent.layerX / MapDrawing.scaleFactor;
					var realY = e.originalEvent.layerY / MapDrawing.scaleFactor;				
					
					// il centro rimane fisso, cambia il punto iniziale
					MapDrawing.startX = MapDrawing.centerX - ((MapDrawing.svgWidth) / this.zoom) / 2;
					MapDrawing.startY = MapDrawing.centerY - ((MapDrawing.svgHeight) / this.zoom) / 2;
					
				} else {
								
					// il centro rimane fisso, cambia il punto iniziale
					MapDrawing.startX = MapDrawing.centerX - ((MapDrawing.svgWidth) / this.zoom) / 2;
					MapDrawing.startY = MapDrawing.centerY - ((MapDrawing.svgHeight) / this.zoom) / 2;
					
				}
				
				// verifico il box
				this.boundBox();
				
				if (this.zoomStyle == 'svg') {
					
					this.draw.viewbox(MapDrawing.startX, 
							MapDrawing.startY, 
							MapDrawing.svgWidth / this.zoom, 
							MapDrawing.svgHeight / this.zoom);
					
				} else {
					$("#map>svg").css("transform","translate(" + 
							(Math.round(MapDrawing.svgWidth / 2) - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom + "px," + 
							(Math.round(MapDrawing.svgHeight / 2) - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom + "px) scale(" + MapDrawing.zoom + ")");
				}
				
				// se ho passato l'evento del mouse 
				// tengo fisse le coordinate del mouse e sposto il centro di conseguenza
				
				if (e && e.originalEvent) {
					
					var Cx = MapDrawing.svgWidth / 2;
					var Cy = MapDrawing.svgHeight / 2;
					var x = ((realX - Cx) / oldZoom - (realX - Cx) / this.zoom) * MapDrawing.scaleFactor * MapDrawing.zoom;
					var y = ((realY - Cy) / oldZoom - (realY - Cy) / this.zoom) * MapDrawing.scaleFactor * MapDrawing.zoom;
										
					this.move(x , y);
				}
				
				this.logReference();
				if (!this.delay) {
					// gestisco i bookmark
					this.evalBookmarks();			
					// aggiorno la minimappa
					this.adjustMini();
				}
				MapDrawing.hideLabel();
			}			
		};
		
		/**
		 * effettua lo zoom out sulla mappa
		 */
		this.zoomOut = function(value, e) {
			
			if (value === undefined) { value = this.zoomMinusFactor; }
			if (e === undefined) { e = null; }
			
			if (this.zoom > this.minzoom && !this.promising) {
				var oldZoom = this.zoom;
				
				if (this.zoom * value > this.minzoom) {
					this.zoom *= value;

					if (e && e.originalEvent) {
						
						// calcolo le coordinate scalate
						var realX = e.originalEvent.layerX / MapDrawing.scaleFactor;
						var realY = e.originalEvent.layerY / MapDrawing.scaleFactor;				
												
						// il centro rimane fisso, cambia il punto iniziale
						MapDrawing.startX = MapDrawing.centerX - (MapDrawing.svgWidth / this.zoom) / 2;
						MapDrawing.startY = MapDrawing.centerY - (MapDrawing.svgHeight / this.zoom) / 2;
						
					} else {
						// il centro rimane fisso, cambia il punto iniziale
						MapDrawing.startX = MapDrawing.centerX - (MapDrawing.svgWidth / this.zoom) / 2;
						MapDrawing.startY = MapDrawing.centerY - (MapDrawing.svgHeight / this.zoom) / 2;
					}
				} else {
					this.zoom = this.minzoom;
					
					// torno al centro e al punto iniziale di origine
					this.centerX = Math.round(this.svgWidth / 2);
					this.centerY = Math.round(this.svgHeight / 2);
					
					MapDrawing.startX = 0;
					MapDrawing.startY = 0;
					
					
				}
				// verifico il box
				this.boundBox();
				
				if (this.zoomStyle == 'svg') {		
					
					this.draw.viewbox(MapDrawing.startX, 
								MapDrawing.startY, 
								MapDrawing.svgWidth / this.zoom, 
								MapDrawing.svgHeight / this.zoom);
					
				} else {
					$("#map>svg").css("transform","translate(" + 
							(Math.round(MapDrawing.svgWidth / 2) - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom + "px," + 
							(Math.round(MapDrawing.svgHeight / 2) - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom + "px) scale(" + MapDrawing.zoom + ")");
				}
				
				// se ho passato l'evento del mouse 
				// tengo fisse le coordinate del mouse e sposto il centro di conseguenza
				
				if (e && e.originalEvent && this.zoom != this.minzoom) {
						
					var Cx = MapDrawing.svgWidth / 2;
					var Cy = MapDrawing.svgHeight / 2;
					var x = ((realX - Cx) / oldZoom - (realX - Cx) / this.zoom) * MapDrawing.scaleFactor * MapDrawing.zoom;
					var y = ((realY - Cy) / oldZoom - (realY - Cy) / this.zoom) * MapDrawing.scaleFactor * MapDrawing.zoom;
						
					this.move(x , y);
				}				
			
				this.logReference();
				if (!this.delay) {
					// gestisco i bookmark
					this.evalBookmarks();
					// aggiorno la minimappa
					this.adjustMini();
				}
				MapDrawing.hideLabel();
			} 
		};
		
		/**
		 * resetta lo zoom sulla mappa
		 */
		this.zoomRestore = function() {
			if (this.zoom > this.minzoom) {				
				this.zoom = 1;
								
				this.centerX = Math.round(this.svgWidth / 2);
				this.centerY = Math.round(this.svgHeight / 2);
				this.startX = 0;
				this.startY = 0;
				
				if (this.zoomStyle == 'svg') {					
					this.draw.viewbox(this.startX, this.startY, this.svgWidth, this.svgHeight);
				} else {
					$("#map>svg").css("transform", "scale("+ this.minzoom +")");
				}
			}
			//console.log('MapDrawing.start: ' + MapDrawing.startX + ', '+ MapDrawing.startY);
			
			// gestisco i bookmark
			this.evalBookmarks();
			// aggiorno la minimappa
			this.adjustMini();
		};			
		
		/**
		 * sposta le coordinate in modo che nel boundingBox non ci sia del bianco
		 */
		this.boundBox = function(x, y) {
			
			if (x === undefined) { x = 0; }
			if (y === undefined) { y = 0; }
			
			var proceed = {};
			proceed.x = true;
			proceed.y = true;
			
			// normalizzo i delta allo stesso modo del move
			x = x / MapDrawing.zoom / MapDrawing.scaleFactor;
			y = y / MapDrawing.zoom / MapDrawing.scaleFactor;
			
			if ((MapDrawing.startX + x) < 0) {
				//console.log('out of bounds left ' );
				MapDrawing.startX = 0;	
				MapDrawing.centerX = MapDrawing.svgWidth / MapDrawing.zoom / 2;
				proceed.x = false;
			}
			
			if ((MapDrawing.startX + x) > (MapDrawing.svgWidth - MapDrawing.svgWidth / MapDrawing.zoom)) {
				//console.log('out of bounds right ' );
				MapDrawing.startX = MapDrawing.svgWidth - MapDrawing.svgWidth / MapDrawing.zoom;
				MapDrawing.centerX = MapDrawing.svgWidth - MapDrawing.svgWidth / MapDrawing.zoom / 2;
				proceed.x = false;
			}
			if ((MapDrawing.startY + y) < 0) {
				//console.log('out of bounds up ' );
				MapDrawing.startY = 0;
				MapDrawing.centerY = MapDrawing.svgHeight / MapDrawing.zoom / 2;
				proceed.y = false;
			}
			
			if ((MapDrawing.startY + y) > (MapDrawing.svgHeight - MapDrawing.svgHeight / MapDrawing.zoom)) {
				//console.log('out of bounds down ' );
				MapDrawing.startY = MapDrawing.svgHeight - MapDrawing.svgHeight / MapDrawing.zoom;
				MapDrawing.centerY = MapDrawing.svgHeight - MapDrawing.svgHeight / MapDrawing.zoom / 2;
				proceed.y = false;
			}
		
			
			return proceed;
		};
		
		/**
		 * muove la mappa
		 * @param int x: offset orizzontale
		 * @param int y: offset verticale
		 * se il parametro  > 0 allora il valore viene sommato al centro
		 * se  < 0 il valore viene sottratto dal centro
		 */
		this.move = function(x,y) {
			if (this.promising) { return; } 
			//console.log('moving ' + x + ', ' + y);
			this.logReference();
			
			// verifico i confini e se procedere con lo spostamento
			var proceed = this.boundBox(x,y);

			if (proceed.x) {
				
				MapDrawing.centerX += x / MapDrawing.scaleFactor / MapDrawing.zoom;
				
				MapDrawing.startX += x / MapDrawing.scaleFactor / MapDrawing.zoom;
								
			} 	
			
			if (proceed.y) {
					MapDrawing.centerY += y / MapDrawing.scaleFactor / MapDrawing.zoom;
					
					MapDrawing.startY += y / MapDrawing.scaleFactor / MapDrawing.zoom;
			}
			
			if (this.zoomStyle == 'svg') {
				MapDrawing.draw.viewbox(MapDrawing.startX, MapDrawing.startY, MapDrawing.svgWidth / MapDrawing.zoom, MapDrawing.svgHeight / MapDrawing.zoom );
			} else {
				$("#map>svg").css("transform","translate(" + 
					(Math.round(MapDrawing.svgWidth / 2) - MapDrawing.centerX)  * MapDrawing.scaleFactor * MapDrawing.zoom + "px," + 
					(Math.round(MapDrawing.svgHeight / 2) - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom + "px) scale(" + MapDrawing.zoom + ")");
			}
			
			this.logReference();
			// gestisco i bookmark
			this.evalBookmarks();
			// aggiorno la minimappa
			this.adjustMini();

			MapDrawing.delay = false;
		};	
		
		/**
		 * visualizza sfondo e disegna i posti in mappa
		 */
		this.paint = function() {
			
			/* versione 3  : 
		 	riscalo il rettangolo di mappa in base alle dimensioni del disegno 
		 	con larghezza massima 1000 e altezza massima 750 */
			
			this.height = Math.floor(this.svgHeight * this.scaleFactor);
			
			var draw = SVG('map').size(this.width, this.height);
			draw.viewbox(0,0,this.svgWidth, this.svgHeight);
			
			var image = draw.image((this.hasroomscreenshot ? '' : '') + Seats.room.picture, this.svgWidth, this.svgHeight);
			image.data("id",'svg-back');
			
			var layer = draw.rect(this.svgWidth, this.svgHeight).fill('transparent').move(0,0);
			layer.data('id', 'svg-layer');
			
			this.draw = draw;
			
			// verifico se devo ridimensionare i box
			if (this.svgHeight < this.svgWidth / 4 * 3) {
				$("#map").css('height', this.height + 2);
				
				// setto l'altezza del map-container
				$("#map-container").css('height',Math.ceil(parseInt($('#shopping-cart').css('height').replace(/px/,'')) + 
							this.height + 2 + 
							parseInt($('#seat-info').css('height').replace('/px/',''))));
				
				
				$('div.sellMap').css('max-height', $('#map-container').css('height'));
				
			}
			
			// se ho dei bookmark parto da quelli e non disegno i seat
			if (Seats.bookmarks.length > 0) {
				for (var i=0; i< Seats.bookmarks.length; i++) {
					var mark = Seats.bookmarks[i];
					mark.index = i;
					
					this.renderBookmark(mark);
					
				}
				
			} else {
				
				// ciclo sui seats e li disegno
				for (var i=0; i< Seats.seats.length; i++) {					
					var seat = Seats.seats[i];

					this.renderSeat(seat);
					
				}
			}
						
			if (this.interactive) {
				// per ogni zona aggiungo l'informazione sulla legenda
				for (var i=0; i<Seats.zones.length; i++) {
					var zone = Seats.zones[i];
					
					if (zone.soldout == false && zone.realseats > 0) {
						var text = document.createElement('div');				
						
						var pic = document.createElement('div');
						pic.style = 'width:1rem; height:1rem; display:inline; ' +
							'background-color:'+zone.color+';margin-right:1rem;' + 
							'float:left; margin-top:2px;border-radius:8px;';				
						
						var tspan = document.createElement('div');				
						tspan.innerHTML = zone.description;
						tspan.appendChild(pic);
						tspan.style = 'font-weight:bold;';
						text.appendChild(tspan);
						
						tspan = document.createElement('div');
						tspan.innerHTML = frasi.mappa['lbl-postiDisponibili'] + zone.available;
						text.appendChild(tspan);
						
						$('#legend-container').append(text);
					}
				}				
			}
			
			MapDrawing.hideLoader();
			//console.log('map displayed');
			
		};
		
		/**
		 * inizializza le dimensioni della mappa e del centro a partire da room
		 */
		this.initSize = function() {			
			this.fixFactor = 750 / Seats.room.height;
			
			// verifico il ratio
			if (Seats.room.ratio == 640 / 480) {
				// calcolo il fattore di scala e il massimo dello zoom
				this.scaleFactor = this.height / Seats.room.height;
				this.svgHeight = parseInt(Seats.room.height);
				this.svgWidth = this.svgHeight / 480 * 640;
				
				
			} else {
				// se l'immagine  wide
				if (Seats.room.ratio > 640 / 480) {
					// prendo la larghezza e scalo l'altezza
					this.scaleFactor = this.width / $('#svg_ref_back').width();
					this.svgWidth = $('#svg_ref_back').width();
					this.svgHeight = $('#svg_ref_back').height();
					this.alertPadding = (40 / (MapDrawing.svgWidth / 640 * 480 / MapDrawing.svgHeight)); 
					$('#map-alert').css('padding-top',(40 / (MapDrawing.svgWidth / 640 * 480 / MapDrawing.svgHeight)) + '%!important');
				}
				// se l'immagine  portrait
				if (Seats.room.ratio < 640 / 480) {
					// prendo l'altezza e scalo la lunghezza
					this.scaleFactor = this.height / $('#svg_ref_back').height();
					this.svgWidth = $('#svg_ref_back').width();
					this.svgHeight = $('#svg_ref_back').height();
					this.adjustX = - (MapDrawing.svgHeight / 480 * 640 - MapDrawing.svgWidth) / 2;
					$('#map-alert').css('padding-top',(40 / (MapDrawing.svgWidth / 640 * 480 / MapDrawing.svgHeight)) + '%!important');
					this.startX = this.adjustX;
				}
				
			}			
			// imposto la larghezza del box posti
			if (Seats.maxSeatDescription > 32) {
				MapDrawing.labelW = 6.8 * Seats.maxSeatDescription;
			}
			
			this.labelW = this.labelW / this.scaleFactor;
			this.labelH = this.labelH / this.scaleFactor;
			this.labelPop = this.labelPop / this.scaleFactor;
			this.labelFont = this.labelFont / this.scaleFactor;
			this.labelPOffset = this.labelPOffset / this.scaleFactor;
			
			this.maxzoom = 8 / this.scaleFactor;
			
			this.centerX = Math.round(this.svgWidth / 2);
			this.centerY = Math.round(this.svgHeight / 2)
			
			
			
		};
		
		/**
		 * gestisce il cambio di dimensioni o di orientamento
		 */
		this.checkOrientation = function(){
	    	//MapDrawing.reset();
	    	MapDrawing.initWindow();
	    	MapDrawing.repaint();
	    	MapDrawing.initSize();
		};

		/**
		 * ridisegna le dimensioni dello sfondo e del viewbox
		 */
		this.repaint = function() {
			
			// svg
			$("svg").attr('width', MapDrawing.width);
			$("svg").attr('height', MapDrawing.height);
		};
		
		/**
		 * inizializza le dimensioni della finestra e delle schede informative
		 */
		this.initWindow = function() {
			// versione interattiva a tutta pagina e con sellMap
			if (this.interactive) {
			
				var padding = parseInt($('.sellMap').parent().css('padding-left').replace(/px/,'')) + 
				parseInt($('.sellMap').parent().css('padding-right').replace(/px/,''));		
			} else {
				// versione non interattiva con legenda laterale e eventMap
				this.abs_height = 360;
				this.abs_width  = 480;
			
				var padding = parseInt($('.eventMap').parent().css('padding-left').replace(/px/,'')) + 
				parseInt($('.eventMap').parent().css('padding-right').replace(/px/,''));
			}
			
			maxHeight = this.abs_height;
			maxWidth = this.abs_width;
			
			
			// se la finestra  pi della larghezza devo tenere conto del padding
			if ($(window).width() >= this.abs_width) {
				maxWidth = this.abs_width;
			} else {
				maxWidth = parseInt($(window).width()) - padding;
			}
			
			var fontH = parseInt($("html").css("font-size").replace(/(px)/,""));
			
			// in base alla larghezza della finestra ricalcolo la larghezza massima dei componenti
			// da notare che il contenitore esterno  al max 640 * 720
			// quando la larghezza supera i 768 la legenda viene affiancata
			// questa riga serve se si fanno le 2 colonne da 8 e 4
			//if ($(window).width() / 3 * 2 > this.abs_width + padding) {
			if ($(window).width() > this.abs_width + padding) {
				maxWidth = this.abs_width;
			} else {
				// qui mi serve considerando anche la scrollbar perch
				// il CSS ne tiene conto 
				if (window.innerWidth > this.break_width) { 
					// questa riga serve se si fanno le 2 colonne da 8 e 4
					//maxWidth = $(window).width() / 3 * 2 - padding;
					maxWidth = $(window).width() - padding;
				} else {
					if ($(window).width() > this.abs_width + padding) {
						maxWidth = this.abs_width;							
					} else {
						maxWidth = $(window).width() - padding;
					}
				}
			}
			
			// setto le dimensioni della mappa
			this.width = maxWidth - 2;
			this.height = Math.floor(maxWidth / 640 * 480) -2;
			
			
			// setto la larghezza di map, seat-info e shopping-cart
			$('#map-container').css('width',maxWidth);
			$("#map").css('width', maxWidth);
			$("#map").css('height', Math.floor((maxWidth) / (this.width + 2) * (this.height+2)));
			$("#shopping-cart").css("width", maxWidth);
			$("#seat-info").css("width", maxWidth);
		
			 		
			if (this.interactive) {
				// ridimensiono cart e legend
				//$('#shopping-cart').css('height', maxHeight  - this.height - parseInt($('#seat-info').css('height').replace(/px/,'')) - 2);
				$('#shopping-cart').css('height', 200 - 2);
				$('#shopping-container').css('height', parseInt($('#shopping-cart').css('height').replace(/px/,'')) - fontH * 5);
				
				// setto l'altezza del map-container
				$("#map-container").css('height',Math.ceil(parseInt($('#shopping-cart').css('height').replace(/px/,'')) + 
							this.height + 2 + 
							parseInt($('#seat-info').css('height').replace('/px/',''))));
				
				
				// setto le dimensioni del sellMap
				$('div.sellMap').css('max-width', maxWidth);
				$('div.sellMap').css('max-height', $('#map-container').css('height'));
			
			} else {
				
				$('#shopping-cart').css('height',0);
				$('#shopping-container').css('height',0);
				$('#map-container').css('height', this.height + 2);

				// setto le dimensioni di eventMap
				$('div.eventMap').css('max-width', maxWidth);
				$('div.eventMap').css('max-height', $('#map-container').css('height'));
			}
			
			//console.log('window : ' + window.innerWidth + ', ' + $(window).height());
			//console.log('map: ' + this.width + ', ' + this.height);			
			
			
		};
		
		
		/**
		 * disegna la label di informazioni posto
		 * se passo il seat devo disegnare la label
		 */
		this.paintLabel = function(seat) {			
			
			if (Mobile()) { return; }
			if (seat === undefined) { seat = null; }
			
			if (seat != null) {
				MapDrawing.currentseat = seat;
			}
			
			if (seat == null) {
				if (MapDrawing.currentseat != null) {
					seat = MapDrawing.currentseat;			
				} else {
					return;
				}
			}
			
			// imposto il prezzo prendendo la prima riduzione 
			// TODO: verificare se  da mettere quella che si chiama Intero
			var price =  Seats.zones[seat.zone].getDefaultPrice().getAmount();
			
			MapDrawing.hideLabel();
						
			// COORDINATE
			var upsidedown = false;
			
			var y = parseInt(seat.y) - (MapDrawing.labelH + MapDrawing.labelPop) / MapDrawing.zoom  ;
			var x = parseInt(seat.x) - MapDrawing.labelW / 2 / MapDrawing.zoom + parseInt(Seats.findType(seat.type).width) / 2 ;
			
			// riposiziono la x
			if (x < MapDrawing.startX + MapDrawing.adjustX / MapDrawing.zoom) {
				x = MapDrawing.startX + MapDrawing.adjustX / MapDrawing.zoom ;
			}
			
			if (x + MapDrawing.labelW / MapDrawing.zoom > MapDrawing.startX + MapDrawing.svgWidth / MapDrawing.zoom) {
				x = MapDrawing.startX + MapDrawing.svgWidth / MapDrawing.zoom - MapDrawing.labelW / MapDrawing.zoom ;
			}
			if (y < MapDrawing.startY) {
				y = parseInt(seat.y) + (MapDrawing.labelPop) / MapDrawing.zoom + parseInt(Seats.findType(seat.type).height)  ;
				upsidedown = true;
			}
			
			// rettangolo contenitore
			var rect = MapDrawing.draw.rect(MapDrawing.labelW / MapDrawing.zoom  ,MapDrawing.labelH / MapDrawing.zoom ).				
				fill('#404040').move(x, y).				
				radius(4 / MapDrawing.scaleFactor / MapDrawing.zoom,4 / MapDrawing.scaleFactor / MapDrawing.zoom);
			
			// testo
			var font = MapDrawing.labelFont / MapDrawing.zoom;
			var text = MapDrawing.draw.text(function(add) {
				add.font({size: font, family: 'arial'});
				  add.tspan(Seats.zones[seat.zone].description).style('font-weight:bold;').fill('#EEE');
				  add.tspan(seat.description).newLine().fill('#EEE');
				  add.tspan(Seats.zones[seat.zone].getDefaultPrice().label).newLine().fill('#EEE');
				  add.tspan('\u20AC ' + price).style('font-weight:bold;').fill('#EEE').dx((MapDrawing.labelW - MapDrawing.labelPOffset - parseInt(Math.log10(price)) * MapDrawing.labelFont * 0.85) / MapDrawing.zoom);
				}).move(x + (4 / MapDrawing.zoom / MapDrawing.scaleFactor), y + (6 / MapDrawing.zoom / MapDrawing.scaleFactor));
		

			// triangolo popup
			var popY = y + (MapDrawing.labelH - 1) / MapDrawing.zoom ;
			//var popX = x + ((MapDrawing.labelW - MapDrawing.labelPop) / 2 / MapDrawing.zoom);
			var popX = parseInt(seat.x) + parseInt(Seats.findType(seat.type).width) / 2 - MapDrawing.labelPop / 2 / MapDrawing.zoom;
			var popPoly = '0,0 ' + (MapDrawing.labelPop / MapDrawing.zoom) + ',0 ' + (MapDrawing.labelPop / 2 / MapDrawing.zoom) + ', ' + (MapDrawing.labelPop  / MapDrawing.zoom);
			if (upsidedown) {
				// popup dal basso all'alto
				popY = y - (MapDrawing.labelPop / MapDrawing.zoom) ;
				popPoly = (MapDrawing.labelPop / 2 / MapDrawing.zoom) + ',0 ' + (MapDrawing.labelPop / MapDrawing.zoom) + ',' + (MapDrawing.labelPop / MapDrawing.zoom) + ' 0,' + (MapDrawing.labelPop / MapDrawing.zoom);
			}
			var pop = MapDrawing.draw.polygon(popPoly).fill('#404040').move(popX, popY);
			
			MapDrawing.labels = MapDrawing.draw.set();
			MapDrawing.labels.add(rect).add(pop).add(text);
		
		};
		
		/**
		 * nasconde la label con le informazioni sul posto
		 */
		this.hideLabel = function() {
			// rimuovo i rettangoli presenti
			if (MapDrawing.labels && MapDrawing.labels.first())  {
				while (MapDrawing.labels.first()) {
					MapDrawing.labels.first().remove();
					MapDrawing.labels.remove(MapDrawing.labels.first());
				}

			}
		};
		
		/**
		 * gestione eventi
		 */
		this.changed = function(event) {

			if (event.target != null) {
				var rect = $("[data-id='" +event.target.id+ "']");
				if (event.target.status >= 0)  {
					switch (event.action) {
						case 'add':
							// cambia colore al posto aggiunto alla selezione
							rect.attr('fill', MapDrawing.selectedColor);
							break;
						case 'remove':
							// cambia colore al posto rimosso dalla selezione
							rect.attr('fill',rect.attr('data-color'));
							break;
						case 'recall':
							// evidenzia il posto in mappa
							MapDrawing.hilite(event.target.index);
							break;
						case 'select':
							//MapDrawing.paintLabel(Seats.seats[rect.data('index')]);							
							break;
					}
				}
			} else {
				if (event.action == 'deselect') {
					MapDrawing.hideLabel();					
				}
			}
		};
		
		
		
		/**
		 * renderizza un posto a sedere in mappa
		 */
		this.renderSeat = function(seat) {
			
			if (Seats.zones[seat.zone] == null) {
				//console.log('unknown zone ' + seat.zone + ' on seat ');
				return;
			}
			var color = Seats.zones[seat.zone].color;
			// se il posto  occupato lo faccio grigio
			if (seat.status != 0) {
				color = MapDrawing.inactiveColor;
			}
			// se il posto  in una zona con restizioni lo faccio in un altro colore
			if (seat.status == -1) {
				color = MapDrawing.restrictedColor;
			}
			// se la zona  soldout lo faccio grigio
			if (Seats.zones[seat.zone].soldout) {
				color = MapDrawing.inactiveColor;
			}
			
			var showcolor = color;
			// se il posto  selezionato lo disegno col colore giusto
			if (Selection.contains(seat.id) !== false) {
				showcolor = MapDrawing.selectedColor;
			}
			//var rect = this.draw.rect(Seats.findType(seat.type).width,
			var rect = this.draw.circle(Seats.findType(seat.type).getSize()).				
				fill(showcolor).
				move(seat.x, seat.y).id('seat_'+seat.index);
		
			rect.data('index', seat.index);			
			if (color != MapDrawing.inactiveColor && color != MapDrawing.restrictedColor && this.interactive) {
				rect.style('cursor','pointer');
				rect.data('id',  seat.id);
				rect.data('color', color);				
			}
			rect.data('mark', seat.bookmark);
			
			if (!Mobile()) {
				if (MapDrawing.interactive) {
					// aggancio l'hover sui posti
					$("#map #seat_"+seat.index).bind('mouseover', function(e) {
						e.preventDefault();
						var seat = Seats.seats[$(this).attr('data-index')];
						if ($(this).get(0).nodeName != 'polygon') {
							if (seat.status == 0 && Seats.zones[seat.zone].soldout == false) {
								// evidenzio in rosso il quadrato
								$(this).attr('fill',MapDrawing.selectedColor);
								
								// creo un rettangolo centrato sul posto con le informazioni
								// zona.descrizione, posto.descrizione, posto.prezzo
							
								if (seat.status == 0) {
									seat.width = $(this).css('width');
									// sparo l'evento di mouse in						
									Selection.current(seat);
									
								} else {
									// sparo l'evento di mouse out
									Selection.current(null);							
								}
							
							}
							if (seat.status == -1) {
								// sparo il messaggio di mouse in					
								Selection.current(seat);
							}
						}
					});						
				
					// aggancio il mouseout sui posti
					$("#map #seat_"+seat.index).bind('mouseout', function(e) {
						e.preventDefault();
						var seat = Seats.seats[$(this).attr('data-index')];
						
						if (seat.status == 0 && Seats.zones[seat.zone].soldout == false) {
							// se non  nella selection ripristino il colore originario, 
							// altrimenti rispristino il colore della selection
							if (Selection.contains($(this).attr('data-id')) !== false) {
								$(this).attr('fill', MapDrawing.selectedColor);							
							} else {
								$(this).attr('fill',$(this).attr('data-color'));
							}
							
						}					
						// sparo l'evento di mouse out				
						Selection.current(null);
		
					});					
				}
			}
			
			if (MapDrawing.interactive) {
				// aggancio il click sui posti
				if (Mobile()) {
					$("#map #seat_"+seat.index).bind('touchstart', function(e) {
						e.preventDefault();
						MapDrawing.toggleSeat($(this));
					});
					
				} else {
					$("#map #seat_"+seat.index).bind('click', function(e) {
						e.preventDefault();
						MapDrawing.toggleSeat($(this));
					});				
				}
			}
		};
		
		/**
		 * evento di click sul posto
		 */
		this.toggleSeat = function(ctrl) {
			
			var seat = Seats.seats[ctrl.data('index')];
			
			if (seat.status == 0  && Seats.zones[seat.zone].soldout == false) {
				Selection.add(seat);
			}
		};
		
		/**
		 * preselezione dei posti se passati da querystring
		 */
		this.preselectSeats = function() {
		
			var query = window.location.search.substring(1);
		    
			if (query.indexOf('seatstructure') > 0) {
				var vars = query.split('&');
			    for (var i = 0; i < vars.length; i++) {
			        var pair = vars[i].split('=');
			        var param = decodeURIComponent(pair[0]);
			        if (param == 'seatstructure') {
			        	pair[1] = pair[1].replace(/\%22/g,'"').replace(/\%7B/g,'{').replace(/\%7D/g,'}');
			        	var value = JSON.parse(pair[1]);
			        	Preselection = value;
			        	
			        	for (var z=0; z<value.seats.length; z++) {
			        		var id = value.seats[z].id;
			        		var rid = value.seats[z].ridId;
			        		var ridindex = -1;
			        		var seat = $('#map').find("[data-id='"+id+"']");
			        		// se lo stato del seat  libero lo posso selezionare
			        		if (Seats.seats[seat.data('index')].status == 0) {
				        		MapDrawing.toggleSeat(seat, true);
				        		seat.trigger("mouseout");
				        		for (var r=0; r<Seats.reductions.length; r++) {
				        			if (Seats.reductions[r].id == rid) {
				        				ridindex = r;
				        			}
				        		}				        		
			        		}
			        	}
			        }
			        
			    }
			}
		
		};
		
		/**
		 * renderizza il bookmark sul settore
		 */
		this.renderBookmark = function(mark) {
			
			if (mark.fake) {
				var color = 'transparent';
				
				// se sono in modalita non interattiva coloro lo sfondo 
				// come il primo posto vuoto che trovo
				if (!this.interactive) {
					color = mark.getSeatColor(); // + 'D0';
				} else {
					color = mark.renderColor() ;
				}
				
				// se  calcolato disegno il poligono
				var poly = this.draw.polygon(mark.getPath()).
				style('cursor','pointer').
				stroke({width: 1 / this.scaleFactor, color: '#3498DB'}).
				fill(color);
				//.move(mark.x, mark.y);

			} else {
				var marksize = 8;
				if (Mobile()) {
					marksize = 4;
				}
				// disegno il cerchio centrato sul settore
				var poly = this.draw.circle(marksize / this.scaleFactor).
				fill (mark.renderColor()).
				style('cursor','pointer').
				move(mark.x + mark.width/2, mark.y + mark.height/2);
				
			}
			poly.data('fill', color);
			poly.data('stroke', mark.renderColor());
			poly.data('name', mark.name);
			poly.data('index', mark.index);
			
			
			// aggancio il mouseover sui path dei settori
			if (Mobile()) {			
				if (MapDrawing.interactive) {
					// aggancio il click sui path dei settori
					$("#map [data-name='"+mark.name+"']").bind('touchstart', function(e) {		
						//if (mark.fake) {
						//	$(this).attr('stroke',$(this).data('stroke') + MapDrawing.shadowColor);
						//} else {
							$(this).attr('fill', MapDrawing.hoverColor);
						//}
						
						// sparo l'evento sul settore
						Selection.bookmark($(this).attr('data-index'));
						
					});
				}
				
				$("#map [data-name='"+mark.name+"']").bind('touchend', function(e) {
					
					// faccio lo zoom per avere i dettagli
					MapDrawing.getBestZoom(mark);
					
					if (mark.fake) {
						// per il rettangolo devo calcolare il centro
						var x = (mark.getCenter('x') - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom;
						var y = (mark.getCenter('y') - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom;
					} else {
						// per il cerchio mi bastano le coordinate
						var x = (parseInt($(this).attr('cx')) - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom;
						var y = (parseInt($(this).attr('cy')) - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom; 
					}

					//console.log("bookmark on "+x+", "+y);
					MapDrawing.promising = false;
					MapDrawing.move(x,y);
					
					if (MapDrawing.interactive) {
						Selection.current(null);
					}
				});
				
			} else {
				
				// aggancio il click sui path dei settori
				$("#map [data-name='"+mark.name+"']").bind('click', function(e) {		
					
					// faccio lo zoom per avere i dettagli
					MapDrawing.getBestZoom(mark);
					
					if (mark.fake) {
						// per il rettangolo devo calcolare il centro
						var x = (mark.getCenter('x') - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom;
						var y = (mark.getCenter('y') - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom;
					} else {
						// per il cerchio mi bastano le coordinate
						var x = (parseInt($(this).attr('cx')) - MapDrawing.centerX) * MapDrawing.scaleFactor * MapDrawing.zoom;
						var y = (parseInt($(this).attr('cy')) - MapDrawing.centerY) * MapDrawing.scaleFactor * MapDrawing.zoom; 
						
					}
					MapDrawing.promising = false;
					MapDrawing.move(x,y);
					
				});
				
				$("#map [data-name='"+mark.name+"']").bind('mouseover', function(e) {
					//if (mark.fake) {
					//	$(this).attr('stroke',$(this).data('stroke') +MapDrawing.shadowColor);
					//} else {
						$(this).attr('fill', MapDrawing.hoverColor);
					//}
					
					if (MapDrawing.interactive) {
						// sparo l'evento sul settore
						Selection.bookmark($(this).attr('data-index'));
					}
	
				});
				// aggancio il mouseout sui path dei settori
				$("#map [data-name='"+mark.name+"']").bind('mouseout', function(e) {
					//if (mark.fake) {
					//	$(this).attr('stroke',$(this).attr('data-stroke'));
					//} else {
						$(this).attr('fill',$(this).attr('data-fill'));
					//}
					if (MapDrawing.interactive) {
						Selection.current(null);
					}
				});
			}
		};
		
		/**
		 * effettua lo zoom sul bookmark per visualizzarlo pi grande possibile
		 */
		this.getBestZoom = function(mark) {
			MapDrawing.delay = true;
			if (mark === undefined) {
				MapDrawing.zoomIn(MapDrawing.switchFactor * 2);
			} else {
				xFactor = MapDrawing.svgWidth / mark.width - 0.5;
				yFactor = MapDrawing.svgHeight / mark.height - 0.5;
				
				MapDrawing.zoomIn(Math.max(Math.min(xFactor, yFactor) / MapDrawing.zoom, MapDrawing.switchFactor / MapDrawing.zoom * 1.25));
			}
		};
		
		/**
		 * verifica se mostrare i bookmark o i posti
		 */
		this.evalBookmarks = function() {
			// se sono arrivato al livello di scala pari a 1 allora posso disegnare i posti del settore
			if (Seats.bookmarks.length > 0 && MapDrawing.zoom >= Math.min(MapDrawing.switchFactor / MapDrawing.fixFactor,MapDrawing.maxzoom * MapDrawing.scaleFactor /2 )) {
				
				MapDrawing.showLoader();
				var promise = MapDrawing.promise();
				// calcolo le coordinate del box che sto mostrando
				showingRect = {};
				
				showingRect.centerX = MapDrawing.centerX;
				showingRect.centerY = MapDrawing.centerY;
				
				showingRect.xa = showingRect.centerX - MapDrawing.svgWidth / MapDrawing.zoom / 2;
				showingRect.xz = showingRect.centerX + MapDrawing.svgWidth / MapDrawing.zoom / 2;
				showingRect.ya = showingRect.centerY - MapDrawing.svgHeight / MapDrawing.zoom / 2;
				showingRect.yz = showingRect.centerY + MapDrawing.svgHeight / MapDrawing.zoom / 2;
							
				// devo usare promise per far s che il loader venga renderizzato
				// in maniera asincrona prima di mostrare i posti e toglierlo
				promise.done(function() {
					
					// calcolo quali settori sto mostrando
					for (var b=0; b<Seats.bookmarks.length; b++) {
						var mark = Seats.bookmarks[b];
						if (mark.isInside(showingRect)) {
							
							// se ho gia renderizzato non faccio nulla
							if (!mark.exploding) {
								// renderizzo i suoi posti a sedere
								for (var i=0; i<mark.seats.length; i++) {								
									var seat = mark.seats[i];
									MapDrawing.renderSeat(seat);
								}
								
								// flagg il bookmark come esploso nei suoi posti
								mark.exploding = true;
								
								// elimino la grafica
								$("#map [data-name='"+mark.name+"']").remove();
							}
							
						} else {
							// se lo stavo mostrando
							if (mark.exploding) {
								
								// nascondo tutti i posti a sedere
								$("#map [data-mark='"+mark.name+"']").remove();
								
								mark.exploding = false;
								MapDrawing.renderBookmark(mark);
								
							}
						}
					}
					MapDrawing.hideLoader();
					MapDrawing.promising = false;
				});
			} else {
				if (Seats.bookmarks.length > 0) {
					// devo ridisegnare i bookmark che non stavo mostrando
					for (var b=0; b<Seats.bookmarks.length; b++) {
						var mark = Seats.bookmarks[b];
						
						// se sto esplodendo i posti
						if (mark.exploding) {
							// nascondo i posti
							$("#map [data-mark='"+mark.name+"']").remove();
							
							mark.exploding = false;
							this.renderBookmark(mark);
						}

					}
					
				}
			}
		};
		
		/*
		 * evidenzia il posto sulla mappa disegnando un cerchio centrato sul posto
		 * che deve pulsare e svanire dopo qualche secondo
		 */
		this.hilite = function(index) {
			// cerco l'elemento dalla struttura dati
			var seat = Seats.seats[index];
			var type = Seats.findType(seat.type);
			var size = 32 / MapDrawing.scaleFactor;			
			
			// faccio uno zoom restore per vedere tutta la pianta
			this.zoomRestore();					
			
			// faccio uno zoom centrato sul posto
			MapDrawing.getBestZoom();
			this.move((seat.x - MapDrawing.centerX ) * (this.scaleFactor * this.zoom), (seat.y - MapDrawing.centerY ) * (this.scaleFactor * this.zoom));

			var promise = MapDrawing.promise();
			
			targetSize = 6 * this.scaleFactor;
			if (Mobile()) {
				targetSize = 2;
			}
			promise.done(function() {
				// creo un cerchio che parte fuori dal posto e si restringe
				var circle1 = MapDrawing.draw.circle(size).
					move(seat.x - (size - type.getSize()) / 2, seat.y - (size - type.getSize()) / 2).
					fill('none').
					stroke({color: MapDrawing.selectedColor, width: targetSize / MapDrawing.scaleFactor});
				circle1.data('ref', index);
				var now = new Date();
				circle1.data('id', index + '-' + now.getMilliseconds());
				//console.log ("hilite seat " + seat.x + ', ' + seat.y);
				
				// animo il cerchio per farlo restringere
				circle1.animate(2500, '>',10).size(0);
				
				MapDrawing.promising = false;
				setTimeout(function() { $("#map>svg>circle[data-id='"+ index + '-' + now.getMilliseconds() +"']").remove()}, 2500);
			});
		};
		
		
		/*
		 * mostra il loader
		 */ 
		this.showLoader = function() {
			$("#map").append("<div class='loader'></div>");
		};
		
		/*
		 * nasconde il loader 
		 */
		this.hideLoader = function() {
			$("#map div.loader").remove();				
		};
		
		this.promise = function() {
			MapDrawing.promising = true;
			var dfd = new $.Deferred();
			$("#map").animate({opacitiy:1}, 10, 
					function() {
				dfd.resolve();
			});
			
			return dfd.promise();
		};
		
		this.logReference = function() {
			var ref = {};
			ref.startX = MapDrawing.startX;
			ref.startY = MapDrawing.startY;
			ref.centerX = MapDrawing.centerX;
			ref.centerY = MapDrawing.centerY;
			ref.scale = MapDrawing.scaleFactor;
			ref.zoom = MapDrawing.zoom;
			ref.xa = ref.centerX - MapDrawing.svgWidth / MapDrawing.zoom / 2;
			ref.xz = ref.centerX + MapDrawing.svgWidth / MapDrawing.zoom / 2;
			ref.ya = ref.centerY - MapDrawing.svgHeight / MapDrawing.zoom / 2;
			ref.yz = ref.centerY + MapDrawing.svgHeight / MapDrawing.zoom / 2;
			
			
			
			//console.log( ref);
		};
		
		/**
		 * crea la minimappa
		 */
		this.toCanvas = function() {
			svg_xml = (new XMLSerializer()).serializeToString($("#map>svg")[0]);
		    var innerImg = svg_xml.match(/(<image){1}.*?(\/>){1}/g);
		    
		    if (MapDrawing.interactive) {
		    	this.minifactor = 6.2;
		    } else {
		    	//this.minifactor = 3.75;
		    	this.minifactor = 2.95;
		    }
		    
		    // this is just a JavaScript (HTML) image
			var div = document.createElement('div');
			div.id = 'mini-container';
			div.className = 'mini-container';
			if (this.svgHeight < this.svgWidth / 4 * 3) {
				div.style.height = (this.svgHeight / this.minifactor) + 'px';				
			}
			
			
			var inImg = new Image();
			tmp = innerImg[0].match(/(href=\")+(.*?)(\")+/);
			inImg.src = tmp[2];
			inImg.style.width = '160px';
			
			if (Seats.room.offsetX != 0) {
				//inImg.style = 'margin-left:' + Seats.room.offsetX + 'px';
			}
			if (Seats.room.offsetY != 0) {
				//inImg.style = 'margin-top:' + Seats.room.offsetY + 'px';
			}
			inImg.className = 'mini-content';
			div.appendChild(inImg);
		
		    var img = new Image();
		    img.id = 'minimap';
		    img.src = "data:image/svg+xml;base64," + btoa(svg_xml.replace(/(image)/g, 'foo'));
		    img.className = 'mini-map mini-content';
		    div.appendChild(img);
			
			var imgLayer = new Image();
			imgLayer.id = 'minilayer';
			imgLayer.className = 'mini-layer mini-content mini-map';
			if (Seats.room.theme == 'dark') {
				imgLayer.style = 'background-color:#E0E0E0;';
			}
			div.appendChild(imgLayer);
			
			$("#map").append(div);
		};
		
		/**
		 * sistemo la minimappa
		 */
		this.adjustMini = function() {
			// su mobile rimane nascosta perch ho poco spazio
			if (!Mobile() && this.hasminimap) {
				// se supero il fattore di switch la mostro
				if (this.zoom < Math.min(this.switchFactor / this.fixFactor,MapDrawing.maxzoom * MapDrawing.scaleFactor /4 )) {
					$('#mini-container').hide();
				} else {
					$('#mini-container').show();
					var wAdjust = (MapDrawing.width / MapDrawing.height * MapDrawing.svgHeight) / MapDrawing.svgWidth;
					// aggiusto lo zoom
					// le dimensioni sono 1/6.2 di 1000 x 750
				
					$("#minilayer").css('width',MapDrawing.width / MapDrawing.minifactor / MapDrawing.zoom );
					$("#minilayer").css('height',MapDrawing.height / MapDrawing.minifactor / MapDrawing.zoom );
					$("#minilayer").css('left', MapDrawing.startX * MapDrawing.scaleFactor / MapDrawing.minifactor * wAdjust);
					$("#minilayer").css('top', MapDrawing.startY * MapDrawing.scaleFactor / MapDrawing.minifactor);
					
				}
			}
		};
		
		/**
		 * segnala un messaggi errore nascondendo mappa e pulsante submit
		 */
		this.signal = function(text, hide) {
			MapDrawing.hideLoader();
			if (hide) {
				$("input:submit[name='btnSubmit']").hide();
				$("#map-container").hide();
			}
			$(".sellMap").append("<h4 class='msg-body bg-danger text-danger'>"+text+"</h4>");
		};
	}	


	
